#!/usr/bin/env python
# -*- coding: utf-8 -*-

#   Ese 3D Python-fu
#   Copyright (C) 2011 Jun Kobayashi
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from gimpfu import *
from decimal import Decimal

def do_ese_3d(img, drawable, dots, flames, ms):
  for i in range(pdb.gimp_image_get_layers(img)[0]):
    pdb.gimp_layer_resize_to_image_size(img.layers[i])

  w = pdb.gimp_image_width(img)
  h = pdb.gimp_image_height(img)
  newimg = pdb.gimp_image_new(w, h, RGB)
  layers = pdb.gimp_image_get_layers(img)
  layers = Decimal(layers[0] - 1)
  gen_flames = int(flames/2) + 1

  for i in range(flames):
    newlayer = pdb.gimp_layer_new(newimg, w, h, RGB_IMAGE, "レイヤー" + str(i) + "(" + str(ms) + "ms)", 100, 0)
    pdb.gimp_image_add_layer(newimg, newlayer, 0)
    pdb.gimp_context_set_background((255, 255, 255))
    pdb.gimp_context_set_foreground((0, 0, 0))
    pdb.gimp_drawable_fill(newlayer, BACKGROUND_FILL)

  for i in range(gen_flames):
    for j in range(layers):
      ratio = Decimal(2)/(layers-Decimal(1))
      if ( j < layers/2 ):
        base_dots = Decimal(dots) * (Decimal(1) - (ratio * Decimal(j)))
        shift_dots = int(( base_dots * gen_flames/2 ) - (base_dots * i))
      else:
        base_dots = Decimal(dots) * (Decimal(1) - (ratio * Decimal(layers - j - 1)))
        shift_dots = int(( base_dots * gen_flames/2 * -1 ) + (base_dots * i))
      pdb.gimp_layer_set_offsets(img.layers[j], shift_dots, 0)
    pdb.gimp_edit_copy_visible(img)
    floating = pdb.gimp_edit_paste(newimg.layers[i], True)
    pdb.gimp_floating_sel_anchor(floating)
    if ( i > 0 ):
      floating = pdb.gimp_edit_paste(newimg.layers[flames-i], True)
      pdb.gimp_floating_sel_anchor(floating)

  for i in range(pdb.gimp_image_get_layers(img)[0]):
    pdb.gimp_layer_set_offsets(img.layers[i], 0, 0)

  pdb.gimp_display_new(newimg)

register(
  "ese_3d",
  "Generate Ese 3D",
  "Generate Ese 3D",
  "Jun Kobayashi",
  "Jun Kobayashi",
  "2011",
  "<Image>/Filters/Animation/Ese 3D...",
  "",
  [(PF_INT, "dots", "1フレーム最大移動ドット数", 3),
   (PF_INT, "flames", "フレーム数", 15),
   (PF_INT, "flames", "フレーム間ウェイト(ミリ秒)", 100)],
  [],
  do_ese_3d)

main()
